@@ -26,8 +26,7 @@
  */
 
 #include <stdlib.h>
-#include <qfile.h>
-#include <qdir.h>
+#include "dir.h"
 #include "doxygen.h"
 #include "outputgen.h"
 #include "parserintf.h"
@@ -48,23 +47,21 @@ class XRefDummyCodeGenerator : public CodeOutputInterface
 
     // these are just null functions, they can be used to produce a syntax highlighted
     // and cross-linked version of the source code, but who needs that anyway ;-)
-    void codify(const char *) {}
-    void writeCodeLink(const char *,const char *,const char *,const char *,const char *)  {}
-    void writeLineNumber(const char *,const char *,const char *,int) {}
-    virtual void writeTooltip(const char *,const DocLinkInfo &,
-                              const char *,const char *,const SourceLinkInfo &,
-                              const SourceLinkInfo &) {}
-    void startCodeLine(bool) {}
-    void endCodeLine() {}
-    void startCodeAnchor(const char *) {}
-    void endCodeAnchor() {}
-    void startFontClass(const char *) {}
-    void endFontClass() {}
-    void writeCodeAnchor(const char *) {}
-    void setCurrentDoc(const Definition *,const char *,bool) {}
-    void addWord(const char *,bool) {}
-    void startCodeFragment(const char *) {}
-    void endCodeFragment(const char *) {}
+    void codify(const QCString &) override {}
+    void writeCodeLink(CodeSymbolType,const QCString &,const QCString &,const QCString &,const QCString &,const QCString &) override  {}
+    void writeLineNumber(const QCString &,const QCString &,const QCString &,int,bool) override {}
+    virtual void writeTooltip(const QCString &,const DocLinkInfo &,
+                              const QCString &,const QCString &,const SourceLinkInfo &,
+                              const SourceLinkInfo &) override {}
+    void startCodeLine(bool) override {}
+    void endCodeLine() override {}
+    void startFontClass(const QCString &) override {}
+    void endFontClass() override {}
+    void writeCodeAnchor(const QCString &) override {}
+    void setCurrentDoc(const Definition *,const QCString &,bool) override {}
+    void addWord(const QCString &,bool) override {}
+    void startCodeFragment(const QCString &) override {}
+    void endCodeFragment(const QCString &) override {}
 
     // here we are presented with the symbols found by the code parser
     void linkableSymbol(int l, const char *sym,Definition *symDef,Definition *context)
@@ -149,7 +146,7 @@ static void listSymbol(Definition *d)
 
 static void listSymbols()
 {
-  for (const auto &kv : Doxygen::symbolMap)
+  for (const auto &kv : *Doxygen::symbolMap)
   {
     listSymbol(kv.second);
   }
@@ -179,8 +176,8 @@ static void lookupSymbol(Definition *d)
       case Definition::TypeFile:
         {
           FileDef *fd = dynamic_cast<FileDef*>(d);
-          printf("Kind: File: #includes %d other files\n",
-              fd->includeFileList() ? fd->includeFileList()->count() : 0);
+          printf("Kind: File: #includes %zu other files\n",
+              fd->includeFileList().size());
         }
         break;
       case Definition::TypeNamespace:
@@ -208,7 +205,7 @@ static void lookupSymbols(const QCString &sym)
 {
   if (!sym.isEmpty())
   {
-    auto range = Doxygen::symbolMap.find(sym);
+    auto range = Doxygen::symbolMap->find(sym);
     bool found=false;
     for (auto it=range.first; it!=range.second; ++it)
     {
@@ -296,11 +293,8 @@ int main(int argc,char **argv)
     }
   }
 
-  // remove temporary files
-  if (!Doxygen::objDBFileName.isEmpty()) QFile::remove(Doxygen::objDBFileName);
-  if (!Doxygen::entryDBFileName.isEmpty()) QFile::remove(Doxygen::entryDBFileName);
   // clean up after us
-  QDir().rmdir("/tmp/doxygen");
+  Dir().rmdir("/tmp/doxygen");
 
   while (1)
   {
