@@ -1,12 +1,10 @@
 /******************************************************************************
  *
- * 
- *
- * Copyright (C) 1997-2015 by Dimitri van Heesch.
+ * Copyright (C) 1997-2019 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -15,9 +13,17 @@
 #include "inputstring.h"
 #include "helplabel.h"
 #include "doxywizard.h"
+#include "config_msg.h"
 #include "config.h"
 
-#include <QtGui>
+#include <QComboBox>
+#include <QLineEdit>
+#include <QGridLayout>
+#include <QWheelEvent>
+#include <QToolBar>
+#include <QFileInfo>
+#include <QFileDialog>
+#include <QTextCodec>
 
 class NoWheelComboBox : public QComboBox
 {
@@ -30,17 +36,19 @@ class NoWheelComboBox : public QComboBox
 
 
 InputString::InputString( QGridLayout *layout,int &row,
-                          const QString & id, const QString &s, 
+                          const QString & id, const QString &s,
                           StringMode m, const QString &docs,
                           const QString &absPath )
   : m_default(s), m_sm(m), m_index(0), m_docs(docs), m_id(id),
-    m_absPath(absPath==QString::fromAscii("1"))
+    m_absPath(absPath==QString::fromLatin1("1"))
 {
   m_lab = new HelpLabel(id);
+  m_brFile = 0;
+  m_brDir = 0;
   if (m==StringFixed)
   {
     layout->addWidget( m_lab, row, 0 );
-    m_com = new NoWheelComboBox; 
+    m_com = new NoWheelComboBox;
     layout->addWidget( m_com, row, 1, 1, 3, Qt::AlignLeft );
     m_le=0;
     m_br=0;
@@ -54,17 +62,17 @@ InputString::InputString( QGridLayout *layout,int &row,
     m_le->setText( s );
     m_im = 0;
     //layout->setColumnMinimumWidth(2,150);
-    if (m==StringFile || m==StringDir || m==StringImage)
+    if (m==StringFile || m==StringDir || m==StringImage || m==StringFileDir)
     {
-      layout->addWidget( m_le, row, 1 );
+      QHBoxLayout *rowLayout = new QHBoxLayout;
+      rowLayout->addWidget( m_le);
       m_br = new QToolBar;
       m_br->setIconSize(QSize(24,24));
-      if (m==StringFile || m==StringImage) 
+      if (m==StringFile || m==StringImage || m==StringFileDir)
       {
-        QAction *file = m_br->addAction(QIcon(QString::fromAscii(":/images/file.png")),QString(),this,SLOT(browse()));
-        file->setToolTip(tr("Browse to a file"));
-        layout->addWidget( m_br,row,2 );
-        if (m==StringImage) 
+        m_brFile = m_br->addAction(QIcon(QString::fromLatin1(":/images/file.png")),QString(),this,SLOT(browseFile()));
+        m_brFile->setToolTip(tr("Browse to a file"));
+        if (m==StringImage)
         {
           m_im = new QLabel;
           m_im->setMinimumSize(1,55);
@@ -73,12 +81,13 @@ InputString::InputString( QGridLayout *layout,int &row,
           layout->addWidget( m_im,row,1 );
         }
       }
-      else 
+      if (m==StringDir || m==StringFileDir)
       {
-        QAction *dir = m_br->addAction(QIcon(QString::fromAscii(":/images/folder.png")),QString(),this,SLOT(browse()));
-        dir->setToolTip(tr("Browse to a folder"));
-        layout->addWidget( m_br,row,2 );
+        m_brDir = m_br->addAction(QIcon(QString::fromLatin1(":/images/folder.png")),QString(),this,SLOT(browseDir()));
+        m_brDir->setToolTip(tr("Browse to a folder"));
       }
+      rowLayout->addWidget( m_br);
+      layout->addLayout( rowLayout, m==StringImage?row-1:row, 1, 1, 2 );
     }
     else
     {
@@ -90,11 +99,11 @@ InputString::InputString( QGridLayout *layout,int &row,
     row++;
   }
 
-  if (m_le)  connect( m_le,   SIGNAL(textChanged(const QString&)), 
+  if (m_le)  connect( m_le,   SIGNAL(textChanged(const QString&)),
                       this,   SLOT(setValue(const QString&)) );
-  if (m_com) connect( m_com,  SIGNAL(activated(const QString &)), 
+  if (m_com) connect( m_com,  SIGNAL(activated(const QString &)),
                       this,   SLOT(setValue(const QString &)) );
-  m_str = s+QChar::fromAscii('!'); // force update
+  m_str = s+QChar::fromLatin1('!'); // force update
   setValue(s);
   connect( m_lab, SIGNAL(enter()), SLOT(help()) );
   connect( m_lab, SIGNAL(reset()), SLOT(reset()) );
@@ -125,11 +134,11 @@ void InputString::updateDefault()
   {
     if (m_str==m_default || !m_lab->isEnabled())
     {
-      m_lab->setText(QString::fromAscii("<qt>")+m_id+QString::fromAscii("</qt"));
+      m_lab->setText(QString::fromLatin1("<qt>")+m_id+QString::fromLatin1("</qt>"));
     }
     else
     {
-      m_lab->setText(QString::fromAscii("<qt><font color='red'>")+m_id+QString::fromAscii("</font></qt>"));
+      m_lab->setText(QString::fromLatin1("<qt><font color='red'>")+m_id+QString::fromLatin1("</font></qt>"));
     }
     if (m_im)
     {
@@ -140,9 +149,9 @@ void InputString::updateDefault()
       else
       {
         QFile Fout(m_str);
-        if(!Fout.exists()) 
+        if(!Fout.exists())
         {
-          m_im->setText(tr("Sorry, cannot find file(")+m_str+QString::fromAscii(");"));
+          m_im->setText(tr("Sorry, cannot find file(")+m_str+QString::fromLatin1(");"));
         }
         else
         {
@@ -153,7 +162,7 @@ void InputString::updateDefault()
           }
           else
           {
-            m_im->setText(tr("Sorry, no preview available (")+m_str+QString::fromAscii(");"));
+            m_im->setText(tr("Sorry, no preview available (")+m_str+QString::fromLatin1(");"));
           }
         }
       }
@@ -169,28 +178,30 @@ void InputString::setEnabled(bool state)
   if (m_le)  m_le->setEnabled(state);
   if (m_im)  m_im->setEnabled(state);
   if (m_br)  m_br->setEnabled(state);
+  if (m_brFile)  m_brFile->setEnabled(state);
+  if (m_brDir)  m_brDir->setEnabled(state);
   if (m_com) m_com->setEnabled(state);
   updateDefault();
 }
 
-void InputString::browse()
+void InputString::browseFile()
 {
   QString path = QFileInfo(MainWindow::instance().configFileName()).path();
-  if (m_sm==StringFile || m_sm==StringImage)
+  QString fileName = QFileDialog::getOpenFileName(&MainWindow::instance(),
+      tr("Select file"),path);
+  if (!fileName.isNull())
   {
-    QString fileName = QFileDialog::getOpenFileName(&MainWindow::instance(),
-        tr("Select file"),path);
-    if (!fileName.isNull()) 
+    QDir dir(path);
+    if (!MainWindow::instance().configFileName().isEmpty() && dir.exists())
     {
-      QDir dir(path);
-      if (!MainWindow::instance().configFileName().isEmpty() && dir.exists())
-      {
-        fileName = m_absPath ? fileName : dir.relativeFilePath(fileName);
-      }
-      setValue(fileName);
+      fileName = m_absPath ? fileName : dir.relativeFilePath(fileName);
     }
+    setValue(fileName);
   }
-  else // sm==StringDir
+}
+void InputString::browseDir()
+{
+  QString path = QFileInfo(MainWindow::instance().configFileName()).path();
   {
     QString dirName = QFileDialog::getExistingDirectory(&MainWindow::instance(),
         tr("Select directory"),path);
@@ -226,7 +237,7 @@ void InputString::setDefault()
   if (index!=-1 && m_com) m_com->setCurrentIndex(index);
 }
 
-QVariant &InputString::value() 
+QVariant &InputString::value()
 {
   return m_value;
 }
@@ -248,3 +259,22 @@ void InputString::writeValue(QTextStream &t,QTextCodec *codec)
   writeStringValue(t,codec,m_str);
 }
 
+bool InputString::isDefault()
+{
+  return m_str == m_default;
+}
+
+QString InputString::checkEnumVal(const QString &value)
+{
+  QString val = value.trimmed().toLower();
+  QStringList::Iterator it;
+  for ( it= m_values.begin(); it != m_values.end(); ++it )
+  {
+    QString enumVal = *it;
+    if (enumVal.toLower() == val) return enumVal;
+  }
+
+  config_warn("argument '%s' for option %s is not a valid enum value."
+              " Using the default: %s!",qPrintable(value),qPrintable(m_id),qPrintable(m_default));
+  return m_default;
+}
