@@ -21,7 +21,7 @@
 #include <string>
 #include <unordered_map>
 
-#include <qcstring.h>
+#include "qcstring.h"
 #include "containers.h"
 
 class FileDef;
@@ -45,7 +45,7 @@ class Define
 };
 
 /** List of all macro definitions */
-using DefineList = std::vector< std::unique_ptr<Define> >;
+using DefineList = std::vector<Define>;
 using DefinesPerFileList = std::unordered_map< std::string, DefineList >;
 
 #endif
