@@ -58,6 +58,8 @@ void DocSets::initialize()
   if (bundleId.isEmpty()) bundleId="org.doxygen.Project";
   QCString feedName = Config_getString(DOCSET_FEEDNAME);
   if (feedName.isEmpty()) feedName="FeedName";
+  QCString feedURL = Config_getString(DOCSET_FEEDURL);
+  if (feedURL.isEmpty()) feedURL="FeedUrl";
   QCString publisherId = Config_getString(DOCSET_PUBLISHER_ID);
   if (publisherId.isEmpty()) publisherId="PublisherId";
   QCString publisherName = Config_getString(DOCSET_PUBLISHER_NAME);
@@ -136,6 +138,8 @@ void DocSets::initialize()
           "     <string>" << projectNumber << "</string>\n"
           "     <key>DocSetFeedName</key>\n"
           "     <string>" << feedName << "</string>\n"
+          "     <key>DocSetFeedUrl</key>\n"
+          "     <string>" << feedURL << "</string>\n"
           "     <key>DocSetPublisherIdentifier</key>\n"
           "     <string>" << publisherId << "</string>\n"
           "     <key>DocSetPublisherName</key>\n"
@@ -257,7 +261,7 @@ void DocSets::addContentsItem(bool isDir,
       }
       else if (!file.isEmpty()) // doxygen generated file
       {
-        p->nts << file << Doxygen::htmlFileExtension;
+        p->nts << addHtmlExtensionIfMissing(file);
       }
       p->nts << "</Path>\n";
       if (!file.isEmpty() && !anchor.isEmpty())
@@ -509,8 +513,7 @@ void DocSets::writeToken(TextStream &t,
     t << "      <Scope>" << convertToXML(scope) << "</Scope>\n";
   }
   t << "    </TokenIdentifier>\n";
-  t << "    <Path>" << d->getOutputFileBase()
-                    << Doxygen::htmlFileExtension << "</Path>\n";
+  t << "    <Path>" << addHtmlExtensionIfMissing(d->getOutputFileBase()) << "</Path>\n";
   if (!anchor.isEmpty())
   {
     t << "    <Anchor>" << anchor << "</Anchor>\n";
