@@ -16,27 +16,25 @@
 #ifndef DOT_H
 #define DOT_H
 
-#include <qlist.h>
-#include <qcstring.h>
 #include <map>
 
-#include "sortdict.h"
-
+#include "qcstring.h"
 #include "dotgraph.h" // only for GraphOutputFormat
 #include "dotfilepatcher.h"
 #include "dotrunner.h"
 
-class FTextStream;
 class DotRunner;
 class DotRunnerQueue;
+class TextStream;
 
 /** Singleton that manages parallel dot invocations and patching files for embedding image maps */
 class DotManager
 {
   public:
     static DotManager *instance();
-    DotRunner*      createRunner(const std::string& absDotName, const std::string& md5Hash);
-    DotFilePatcher *createFilePatcher(const std::string &fileName);
+    static void deleteInstance();
+    DotRunner*      createRunner(const QCString& absDotName, const QCString& md5Hash);
+    DotFilePatcher *createFilePatcher(const QCString &fileName);
     bool run() const;
 
   private:
@@ -47,14 +45,16 @@ class DotManager
     std::map<std::string, DotFilePatcher>  m_filePatchers;
     static DotManager     *m_theInstance;
     DotRunnerQueue        *m_queue;
-    QList<DotWorkerThread> m_workers;
+    std::vector< std::unique_ptr<DotWorkerThread> > m_workers;
 };
 
-void writeDotGraphFromFile(const char *inFile,const char *outDir,
-                           const char *outFile,GraphOutputFormat format);
-void writeDotImageMapFromFile(FTextStream &t,
-                              const QCString& inFile, const QCString& outDir,
-                              const QCString& relPath,const QCString& baseName,
-                              const QCString& context,int graphId=-1);
+void writeDotGraphFromFile(const QCString &inFile,const QCString &outDir,
+                           const QCString &outFile,GraphOutputFormat format,
+                           const QCString &srcFile,int srcLine);
+void writeDotImageMapFromFile(TextStream &t,
+                              const QCString &inFile, const QCString& outDir,
+                              const QCString &relPath,const QCString& baseName,
+                              const QCString &context,int graphId,
+                              const QCString &srcFile,int srcLine);
 
 #endif
