@@ -16,7 +16,6 @@
 #ifndef DOTRUNNER_H
 #define DOTRUNNER_H
 
-#include <qglobal.h> //uint
 #include <string>
 #include <thread>
 #include <list>
@@ -25,26 +24,31 @@
 #include <condition_variable>
 #include <memory>
 
+#include "qcstring.h"
+
 /** Helper class to run dot from doxygen from multiple threads.  */
 class DotRunner
 {
     struct DotJob
     {
-      DotJob(std::string f, std::string o, std::string a)
-        : format(f), output(o), args(a) {}
-      std::string format;
-      std::string output;
-      std::string args;
+      DotJob(const QCString &f, const QCString &o, const QCString &a,
+             const QCString &s,int l)
+        : format(f), output(o), args(a), srcFile(s), srcLine(l) {}
+      QCString format;
+      QCString output;
+      QCString args;
+      QCString srcFile;
+      int srcLine;
     };
 
   public:
     /** Creates a runner for a dot \a file. */
-    DotRunner(const std::string& absDotName, const std::string& md5Hash = std::string());
+    DotRunner(const QCString & absDotName, const QCString& md5Hash = QCString());
 
     /** Adds an additional job to the run.
      *  Performing multiple jobs one file can be faster.
      */
-    void addJob(const char *format,const char *output);
+    void addJob(const QCString &format,const QCString &output,const QCString &srcFile,int srcLine);
 
     /** Prevent cleanup of the dot file (for user provided dot files) */
     void preventCleanUp() { m_cleanUp = false; }
@@ -52,16 +56,15 @@ class DotRunner
     /** Runs dot for all jobs added. */
     bool run();
 
-    //  DotConstString const& getFileName() { return m_file; }
-    std::string const & getMd5Hash() { return m_md5Hash; }
+    QCString getMd5Hash() { return m_md5Hash; }
 
-    static bool readBoundingBox(const char* fileName, int* width, int* height, bool isEps);
+    static bool readBoundingBox(const QCString &fileName, int* width, int* height, bool isEps);
 
   private:
-    std::string m_file;
-    std::string m_md5Hash;
-    std::string m_dotExe;
-    bool        m_cleanUp;
+    QCString m_file;
+    QCString m_md5Hash;
+    QCString m_dotExe;
+    bool     m_cleanUp;
     std::vector<DotJob>  m_jobs;
 };
 
@@ -72,7 +75,7 @@ class DotRunnerQueue
   public:
     void enqueue(DotRunner *runner);
     DotRunner *dequeue();
-    uint count() const;
+    size_t size() const;
   private:
     std::condition_variable m_bufferNotEmpty;
     std::queue<DotRunner *> m_queue;
@@ -80,10 +83,11 @@ class DotRunnerQueue
 };
 
 /** Worker thread to execute a dot run */
-class DotWorkerThread 
+class DotWorkerThread
 {
   public:
     DotWorkerThread(DotRunnerQueue *queue);
+   ~DotWorkerThread();
     void run();
     void start();
     bool isRunning() { return m_thread && m_thread->joinable(); }
