@@ -1,12 +1,12 @@
 /******************************************************************************
  *
- * 
+ *
  *
  * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -16,26 +16,24 @@
  */
 
 #include <algorithm>
+#include <atomic>
 #include <stdlib.h>
-#include <qfile.h>
+
 #include "entry.h"
 #include "util.h"
 #include "section.h"
 #include "doxygen.h"
 #include "arguments.h"
 #include "config.h"
-//------------------------------------------------------------------
-
-#define HEADER ('D'<<24)+('O'<<16)+('X'<<8)+'!'
 
 //------------------------------------------------------------------
 
-int Entry::num=0;
+static AtomicInt g_num;
 
 Entry::Entry()
 {
   //printf("Entry::Entry(%p)\n",this);
-  num++;
+  g_num++;
   m_parent=0;
   section = EMPTY_SEC;
   //printf("Entry::Entry() tArgList=0\n");
@@ -50,7 +48,7 @@ Entry::Entry()
 Entry::Entry(const Entry &e)
 {
   //printf("Entry::Entry(%p):copy\n",this);
-  num++;
+  g_num++;
   section     = e.section;
   type        = e.type;
   name        = e.name;
@@ -74,8 +72,8 @@ Entry::Entry(const Entry &e)
   bitfields   = e.bitfields;
   argList     = e.argList;
   tArgLists   = e.tArgLists;
-  program     = e.program;
-  initializer = e.initializer;
+  program.str(e.program.str());
+  initializer.str(e.initializer.str());
   includeFile = e.includeFile;
   includeName = e.includeName;
   doc         = e.doc;
@@ -110,6 +108,7 @@ Entry::Entry(const Entry &e)
   id          = e.id;
   extends     = e.extends;
   groups      = e.groups;
+  req         = e.req;
   m_fileDef   = e.m_fileDef;
 
   m_parent    = e.m_parent;
@@ -123,11 +122,11 @@ Entry::Entry(const Entry &e)
 
 Entry::~Entry()
 {
-  //printf("Entry::~Entry(%p) num=%d\n",this,num);
+  //printf("Entry::~Entry(%p) num=%d\n",this,g_num);
   //printf("Deleting entry %d name %s type %x children %d\n",
-  //       num,name.data(),section,sublist->count());
+  //       num,qPrint(name),section,sublist->count());
 
-  num--;
+  g_num--;
 }
 
 void Entry::moveToSubEntryAndRefresh(Entry *&current)
@@ -150,7 +149,7 @@ void Entry::moveToSubEntryAndKeep(Entry *current)
   m_sublist.emplace_back(current);
 }
 
-void Entry::moveToSubEntryAndKeep(std::shared_ptr<Entry> &current)
+void Entry::moveToSubEntryAndKeep(std::shared_ptr<Entry> current)
 {
   current->m_parent=this;
   m_sublist.push_back(current);
@@ -183,17 +182,17 @@ void Entry::removeSubEntry(const Entry *e)
 
 void Entry::reset()
 {
-  static bool entryCallGraph   = Config_getBool(CALL_GRAPH);
-  static bool entryCallerGraph = Config_getBool(CALLER_GRAPH);
-  static bool entryReferencedByRelation = Config_getBool(REFERENCED_BY_RELATION);
-  static bool entryReferencesRelation   = Config_getBool(REFERENCES_RELATION);
+  bool entryCallGraph   = Config_getBool(CALL_GRAPH);
+  bool entryCallerGraph = Config_getBool(CALLER_GRAPH);
+  bool entryReferencedByRelation = Config_getBool(REFERENCED_BY_RELATION);
+  bool entryReferencesRelation   = Config_getBool(REFERENCES_RELATION);
   //printf("Entry::reset()\n");
   name.resize(0);
   type.resize(0);
   args.resize(0);
   bitfields.resize(0);
   exception.resize(0);
-  program.resize(0);
+  program.str(std::string());
   includeFile.resize(0);
   includeName.resize(0);
   doc.resize(0);
@@ -209,7 +208,7 @@ void Entry::reset()
   inbodyLine=-1;
   inside.resize(0);
   fileName.resize(0);
-  initializer.resize(0);
+  initializer.str(std::string());
   initLines = -1;
   startLine = 1;
   startColumn = 1;
@@ -240,11 +239,11 @@ void Entry::reset()
   extends.clear();
   groups.clear();
   anchors.clear();
-  argList.clear();
-  tArgLists.clear();
   argList.reset();
+  tArgLists.clear();
   typeConstr.reset();
   sli.clear();
+  req.resize(0);
   m_fileDef = 0;
 }
 
