@@ -1,13 +1,13 @@
 /******************************************************************************
  *
- * 
+ *
  *
  *
  * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -16,26 +16,27 @@
  *
  */
 
-#ifndef _MANDOCVISITOR_H
-#define _MANDOCVISITOR_H
+#ifndef MANDOCVISITOR_H
+#define MANDOCVISITOR_H
 
+#include <iostream>
+
+#include "qcstring.h"
 #include "docvisitor.h"
-#include <qstack.h>
-#include <qcstring.h>
 
-class FTextStream;
 class CodeOutputInterface;
+class TextStream;
 
 /*! @brief Concrete visitor implementation for LaTeX output. */
 class ManDocVisitor : public DocVisitor
 {
   public:
-    ManDocVisitor(FTextStream &t,CodeOutputInterface &ci,const char *langExt);
-    
+    ManDocVisitor(TextStream &t,CodeOutputInterface &ci,const QCString &langExt);
+
     //--------------------------------------
     // visitor functions for leaf nodes
     //--------------------------------------
-    
+
     void visit(DocWord *);
     void visit(DocLinkedWord *);
     void visit(DocWhiteSpace *);
@@ -57,7 +58,7 @@ class ManDocVisitor : public DocVisitor
     //--------------------------------------
     // visitor functions for compound nodes
     //--------------------------------------
-    
+
     void visitPre(DocAutoList *);
     void visitPost(DocAutoList *);
     void visitPre(DocAutoListItem *);
@@ -128,8 +129,6 @@ class ManDocVisitor : public DocVisitor
     void visitPost(DocXRefItem *);
     void visitPre(DocInternalRef *);
     void visitPost(DocInternalRef *);
-    void visitPre(DocCopy *);
-    void visitPost(DocCopy *);
     void visitPre(DocText *);
     void visitPost(DocText *);
     void visitPre(DocHtmlBlockQuote *);
@@ -142,26 +141,31 @@ class ManDocVisitor : public DocVisitor
   private:
 
     //--------------------------------------
-    // helper functions 
+    // helper functions
     //--------------------------------------
-    
-    void filter(const char *str);
 
-    void pushEnabled();
-    void popEnabled();
+    void filter(const QCString &str);
 
     //--------------------------------------
     // state variables
     //--------------------------------------
 
-    FTextStream &m_t;
+    TextStream &m_t;
     CodeOutputInterface &m_ci;
     bool m_insidePre;
     bool m_hide;
     bool m_firstCol;
     int  m_indent;
-    QStack<bool> m_enabled;
     QCString m_langExt;
 };
 
+struct ManListItemInfo
+{
+  int number;
+  char type;
+};
+
+const int man_maxIndentLevels = 13;
+
+extern ManListItemInfo man_listItemInfo[man_maxIndentLevels];
 #endif
