@@ -1,13 +1,13 @@
 /******************************************************************************
  *
- * 
+ *
  *
  *
  * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -16,26 +16,24 @@
  *
  */
 
-#ifndef _TEXTDOCVISITOR_H
-#define _TEXTDOCVISITOR_H
+#ifndef TEXTDOCVISITOR_H
+#define TEXTDOCVISITOR_H
 
+#include "qcstring.h"
 #include "docvisitor.h"
 #include "docparser.h"
-#include <qstack.h>
-#include <qcstring.h>
-#include "ftextstream.h"
-
+#include "textstream.h"
 
 /*! @brief Concrete visitor implementation for TEXT output. */
 class TextDocVisitor : public DocVisitor
 {
   public:
-    TextDocVisitor(FTextStream &t) : DocVisitor(DocVisitor_Text), m_t(t) {}
-    
+    TextDocVisitor(TextStream &t) : DocVisitor(DocVisitor_Text), m_t(t) {}
+
     //--------------------------------------
     // visitor functions for leaf nodes
     //--------------------------------------
-    
+
     void visit(DocWord *w)        { filter(w->word()); }
     void visit(DocLinkedWord *w)  { filter(w->word()); }
     void visit(DocWhiteSpace *)   { m_t << " ";       }
@@ -57,7 +55,7 @@ class TextDocVisitor : public DocVisitor
     //--------------------------------------
     // visitor functions for compound nodes
     //--------------------------------------
-    
+
     void visitPre(DocAutoList *) {}
     void visitPost(DocAutoList *) {}
     void visitPre(DocAutoListItem *) {}
@@ -125,8 +123,6 @@ class TextDocVisitor : public DocVisitor
     void visitPost(DocXRefItem *) {}
     void visitPre(DocInternalRef *) {}
     void visitPost(DocInternalRef *) {}
-    void visitPre(DocCopy *) {}
-    void visitPost(DocCopy *) {}
     void visitPre(DocText *) {}
     void visitPost(DocText *) {}
     void visitPre(DocHtmlBlockQuote *) {}
@@ -138,9 +134,9 @@ class TextDocVisitor : public DocVisitor
 
   private:
 
-   void filter(const char *str);
+    void filter(const QCString &str);
 
-    FTextStream &m_t;
+    TextStream &m_t;
 };
 
 #endif
