@@ -20,7 +20,6 @@
 
 #include "classdef.h"
 #include "config.h"
-#include "vhdldocgen.h"
 
 /** Abstract base class for all translatable text fragments. */
 class Translator
@@ -31,7 +30,7 @@ class Translator
      *  if the user chooses a language whose translation is not up to date.
      *  It is implemented by the adapter classes.
      */
-    virtual QCString updateNeededMessage() { return ""; }
+    virtual QCString updateNeededMessage() { return QCString(); }
     virtual ~Translator() {}
 
     // Please, have a look at comments inside the translator_en.h file
@@ -49,17 +48,22 @@ class Translator
      * this routine. In case no font encoding is required the empty string
      * can be returned.
      */
-    virtual QCString latexFontenc() { return "T1"; }
+    virtual QCString latexFontenc() { return QCString("T1"); }
+    virtual QCString latexFont() {
+      return QCString("\\usepackage[scaled=.90]{helvet}\n"
+             "\\usepackage{courier}\n"
+             "\\renewcommand{\\familydefault}{\\sfdefault}\n");
+    }
     /*!
-     * Sets the commands to be insered directly after the `\\begin{document}`
+     * Sets the commands to be inserted directly after the `\\begin{document}`
      * in the LaTeX document.
      */
-    virtual QCString latexDocumentPre() { return ""; }
+    virtual QCString latexDocumentPre() { return QCString(""); }
     /*!
-     * Sets the commands to be insered directly before the `\\end{document}`
+     * Sets the commands to be inserted directly before the `\\end{document}`
      * in the LaTeX document.
      */
-    virtual QCString latexDocumentPost() { return ""; }
+    virtual QCString latexDocumentPost() { return QCString(""); }
     /*!
      * Set the name to be used as latex command.
      */
@@ -73,6 +77,7 @@ class Translator
       }
       return latex_command;
     }
+    virtual QCString trISOLang() = 0;
 
     // --- Language translation methods -------------------
 
@@ -88,7 +93,7 @@ class Translator
     virtual QCString trMemberList() = 0;
     virtual QCString trThisIsTheListOfAllMembers() = 0;
     virtual QCString trIncludingInheritedMembers() = 0;
-    virtual QCString trGeneratedAutomatically(const char *s) = 0;
+    virtual QCString trGeneratedAutomatically(const QCString &s) = 0;
     virtual QCString trEnumName() = 0;
     virtual QCString trEnumValue() = 0;
     virtual QCString trDefinedIn() = 0;
@@ -143,9 +148,9 @@ class Translator
     virtual QCString trFunctionDocumentation() = 0;
     virtual QCString trVariableDocumentation() = 0;
     virtual QCString trCompounds() = 0;
-    virtual QCString trGeneratedAt(const char *date,const char *projName) = 0;
+    virtual QCString trGeneratedAt(const QCString &date,const QCString &projName) = 0;
     //virtual QCString trWrittenBy() = 0;
-    virtual QCString trClassDiagram(const char *clName) = 0;
+    virtual QCString trClassDiagram(const QCString &clName) = 0;
     virtual QCString trForInternalUseOnly() = 0;
     //virtual QCString trReimplementedForInternalReasons() = 0;
     virtual QCString trWarning() = 0;
@@ -176,12 +181,12 @@ class Translator
 // new since 0.49-990425
 //////////////////////////////////////////////////////////////////////////
 
-    virtual QCString trCompoundReference(const char *clName,
+    virtual QCString trCompoundReference(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate) = 0;
 
-    virtual QCString trFileReference(const char *fileName) = 0;
-    virtual QCString trNamespaceReference(const char *namespaceName) = 0;
+    virtual QCString trFileReference(const QCString &fileName) = 0;
+    virtual QCString trNamespaceReference(const QCString &namespaceName) = 0;
 
     virtual QCString trPublicMembers() = 0;
     virtual QCString trPublicSlots() = 0;
@@ -243,8 +248,8 @@ class Translator
 // new since 1.0.0
 //////////////////////////////////////////////////////////////////////////
 
-    virtual QCString trCollaborationDiagram(const char *clName) = 0;
-    virtual QCString trInclDepGraph(const char *fName) = 0;
+    virtual QCString trCollaborationDiagram(const QCString &clName) = 0;
+    virtual QCString trInclDepGraph(const QCString &fName) = 0;
     virtual QCString trConstructorDocumentation() = 0;
     virtual QCString trGotoSourceCode() = 0;
     virtual QCString trGotoDocumentation() = 0;
@@ -325,7 +330,7 @@ class Translator
 
     //virtual QCString trInterfaces() = 0;
     virtual QCString trClasses() = 0;
-    virtual QCString trPackage(const char *name) = 0;
+    virtual QCString trPackage(const QCString &name) = 0;
     virtual QCString trPackageList() = 0;
     virtual QCString trPackageListDescription() = 0;
     virtual QCString trPackages() = 0;
@@ -491,7 +496,7 @@ class Translator
     virtual QCString trDirDocumentation() = 0;
     virtual QCString trDirectories() = 0;
     virtual QCString trDirDescription() = 0;
-    virtual QCString trDirReference(const char *dirName) = 0;
+    virtual QCString trDirReference(const QCString &dirName) = 0;
     virtual QCString trDir(bool first_capital, bool singular) = 0;
 
 //////////////////////////////////////////////////////////////////////////
@@ -523,10 +528,10 @@ class Translator
     virtual QCString trDataTypes() = 0;
     virtual QCString trModulesList() = 0;
     virtual QCString trModulesListDescription(bool extractAll) = 0;
-    virtual QCString trCompoundReferenceFortran(const char *clName,
+    virtual QCString trCompoundReferenceFortran(const QCString &clName,
                                  ClassDef::CompoundType compType,
                                  bool isTemplate) = 0;
-    virtual QCString trModuleReference(const char *namespaceName) = 0;
+    virtual QCString trModuleReference(const QCString &namespaceName) = 0;
     virtual QCString trModulesMembers() = 0;
     virtual QCString trModulesMemberDescription(bool extractAll) = 0;
     virtual QCString trModulesIndex() = 0;
@@ -541,7 +546,7 @@ class Translator
 // new since 1.6.0
 //////////////////////////////////////////////////////////////////////////
 
-    virtual QCString trDirRelation(const char *name) = 0;
+    virtual QCString trDirRelation(const QCString &name) = 0;
     virtual QCString trLoading() = 0;
     virtual QCString trGlobalNamespace() = 0;
     virtual QCString trSearching() = 0;
@@ -551,9 +556,9 @@ class Translator
 // new since 1.6.3
 //////////////////////////////////////////////////////////////////////////
 
-    //virtual QCString trDirDependency(const char *name) = 0;
-    virtual QCString trFileIn(const char *name) = 0;
-    virtual QCString trIncludesFileIn(const char *name) = 0;
+    //virtual QCString trDirDependency(const QCString &name) = 0;
+    virtual QCString trFileIn(const QCString &name) = 0;
+    virtual QCString trIncludesFileIn(const QCString &name) = 0;
     virtual QCString trDateTime(int year,int month,int day,int dayOfWeek,
                                 int hour,int minutes,int seconds,
                                 bool includeTime) = 0;
@@ -564,7 +569,7 @@ class Translator
 
     virtual QCString trCiteReferences() = 0;
     virtual QCString trCopyright() = 0;
-    virtual QCString trDirDepGraph(const char *name) = 0;
+    virtual QCString trDirDepGraph(const QCString &name) = 0;
 
 //////////////////////////////////////////////////////////////////////////
 // new since 1.8.0
@@ -574,8 +579,8 @@ class Translator
     virtual QCString trTemplateParameters() = 0;
     virtual QCString trAndMore(const QCString &number) = 0;
     virtual QCString trEnumGeneratedFromFiles(bool single) = 0;
-    virtual QCString trEnumReference(const char *name) = 0;
-    virtual QCString trInheritedFrom(const char *members,const char *what) = 0;
+    virtual QCString trEnumReference(const QCString &name) = 0;
+    virtual QCString trInheritedFrom(const QCString &members,const QCString &what) = 0;
     virtual QCString trAdditionalInheritedMembers() = 0;
 
 //////////////////////////////////////////////////////////////////////////
@@ -588,7 +593,6 @@ class Translator
     virtual QCString trClassMethods() = 0;
     virtual QCString trInstanceMethods() = 0;
     virtual QCString trMethodDocumentation() = 0;
-    virtual QCString trDesignOverview() = 0;
 
 //////////////////////////////////////////////////////////////////////////
 // new since 1.8.4
@@ -597,9 +601,9 @@ class Translator
     virtual QCString trInterfaces() = 0;
     virtual QCString trServices() = 0;
     virtual QCString trConstantGroups() = 0;
-    virtual QCString trConstantGroupReference(const char *namespaceName) = 0;
-    virtual QCString trServiceReference(const char *sName) = 0;
-    virtual QCString trSingletonReference(const char *sName) = 0;
+    virtual QCString trConstantGroupReference(const QCString &namespaceName) = 0;
+    virtual QCString trServiceReference(const QCString &sName) = 0;
+    virtual QCString trSingletonReference(const QCString &sName) = 0;
     virtual QCString trServiceGeneratedFromFiles(bool single) = 0;
     virtual QCString trSingletonGeneratedFromFiles(bool single) = 0;
 
@@ -615,7 +619,7 @@ class Translator
     virtual QCString trDesignUnits() = 0;
     virtual QCString trFunctionAndProc() = 0;
     virtual QCString trVhdlType(uint64 type,bool single) = 0;
-    virtual QCString trCustomReference(const char *name) = 0;
+    virtual QCString trCustomReference(const QCString &name) = 0;
 
     virtual QCString trConstants() = 0;
     virtual QCString trConstantDocumentation() = 0;
@@ -641,11 +645,29 @@ class Translator
     virtual QCString trExceptionHierarchy() = 0;
     virtual QCString trExceptionHierarchyDescription() = 0;
     virtual QCString trExceptionDocumentation() = 0;
-    virtual QCString trCompoundReferenceSlice(const char *clName, ClassDef::CompoundType compType, bool isLocal) = 0;
+    virtual QCString trCompoundReferenceSlice(const QCString &clName, ClassDef::CompoundType compType, bool isLocal) = 0;
     virtual QCString trOperations() = 0;
     virtual QCString trOperationDocumentation() = 0;
     virtual QCString trDataMembers() = 0;
     virtual QCString trDataMemberDocumentation() = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.19
+//////////////////////////////////////////////////////////////////////////
+
+    virtual QCString trDesignUnitDocumentation() = 0;
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.9.2
+//////////////////////////////////////////////////////////////////////////
+
+    virtual QCString trConcept(bool first_capital, bool singular) = 0;
+    virtual QCString trConceptReference(const QCString &conceptName) = 0;
+    virtual QCString trConceptList() = 0;
+    virtual QCString trConceptIndex() = 0;
+    virtual QCString trConceptDocumentation() = 0;
+    virtual QCString trConceptListDescription(bool extractAll) = 0;
+    virtual QCString trConceptDefinition() = 0;
 };
 
 #endif
