@@ -32,9 +32,11 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
     /* Used to get the command(s) for the language support. */
     virtual QCString latexLanguageSupportCommand()
     {
-        return "\\usepackage[latin]{armtex}\n"
-	       "\\usepackage[armscii8]{inputenc}\n";
+      return "\\usepackage[latin]{armtex}\n"
+             "\\usepackage[armscii8]{inputenc}\n";
     }
+    virtual QCString trISOLang()
+    { return "hy"; }
 
     // --- Language translation methods -------------------
 
@@ -106,9 +108,9 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
     /*! this is put at the author sections at the bottom of man pages.
      *  parameter s is name of the project name.
      */
-    virtual QCString trGeneratedAutomatically(const char *s)
+    virtual QCString trGeneratedAutomatically(const QCString &s)
     { QCString result="Ավտոմատ ստեղծված է ելքային կոդից, Doxygen-ի միջոցով, ";
-      if (s) result+=s+(QCString)" համար:";
+      if (!s.isEmpty()) result+=s+(QCString)" համար:";
       return result;
     }
 
@@ -455,16 +457,16 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
     /*! This is used in the standard footer of each page and indicates when
      *  the page was generated
      */
-    virtual QCString trGeneratedAt(const char *date,const char *projName)
+    virtual QCString trGeneratedAt(const QCString &date,const QCString &projName)
     {
       QCString result=(QCString)"Ստեղծվել է "+date;
-      if (projName) result+=projName+QCString(" -ի համար,");
+      if (!projName.isEmpty()) result+=projName+QCString(" -ի համար,");
       result+=(QCString)" հետևյալ համակարգով.";
       return result;
     }
 
     /*! this text is put before a class diagram */
-    virtual QCString trClassDiagram(const char *clName)
+    virtual QCString trClassDiagram(const QCString &clName)
     {
       return clName+QCString(" -ի ժառանգման գծագիրը.");
     }
@@ -543,7 +545,7 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
 //////////////////////////////////////////////////////////////////////////
 
     /*! used as the title of the HTML page of a class/struct/union */
-    virtual QCString trCompoundReference(const char *clName,
+    virtual QCString trCompoundReference(const QCString &clName,
                                  ClassDef::CompoundType compType,
                                  bool isTemplate)
     {
@@ -581,13 +583,13 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
     }
 
     /*! used as the title of the HTML page of a file */
-    virtual QCString trFileReference(const char *fileName)
+    virtual QCString trFileReference(const QCString &fileName)
     {
       return fileName+QCString(" ֆայլեր");
     }
 
     /*! used as the title of the HTML page of a namespace */
-    virtual QCString trNamespaceReference(const char *namespaceName)
+    virtual QCString trNamespaceReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+=" անունների տարածություններ";
@@ -785,12 +787,12 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
 //////////////////////////////////////////////////////////////////////////
 
     /*! this text is put before a collaboration diagram */
-    virtual QCString trCollaborationDiagram(const char *clName)
+    virtual QCString trCollaborationDiagram(const QCString &clName)
     {
       return (QCString)clName+"-ի համագործակցությունների գծագիր.";
     }
     /*! this text is put before an include dependency graph */
-    virtual QCString trInclDepGraph(const char *fName)
+    virtual QCString trInclDepGraph(const QCString &fName)
     {
       return (QCString)fName+"-ի ներառումների կախվածությունների գծագիր.";
     }
@@ -1072,7 +1074,7 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
       }
     }
     /*! Used as the title of a Java package */
-    virtual QCString trPackage(const char *name)
+    virtual QCString trPackage(const QCString &name)
     {
       return (QCString)"Փաթեթ "+name;
     }
@@ -1444,7 +1446,7 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
     /*! This returns the title of a directory page. The name of the
      *  directory is passed via \a dirName.
      */
-    virtual QCString trDirReference(const char *dirName)
+    virtual QCString trDirReference(const QCString &dirName)
     { QCString result=dirName; result+=" Ֆայլադարան"; return result; }
 
     /*! This returns the word directory with or without starting capital
@@ -1570,7 +1572,7 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
     }
 
     /*! used as the title of the HTML page of a module/type (Fortran) */
-    virtual QCString trCompoundReferenceFortran(const char *clName,
+    virtual QCString trCompoundReferenceFortran(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
@@ -1607,7 +1609,7 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
       return result;
     }
     /*! used as the title of the HTML page of a module (Fortran) */
-    virtual QCString trModuleReference(const char *namespaceName)
+    virtual QCString trModuleReference(const QCString &namespaceName)
     {
       return QCString("Մոդուլ ") + namespaceName;
     }
@@ -1703,7 +1705,7 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
 //////////////////////////////////////////////////////////////////////////
 
     /*! directory relation for \a name */
-    virtual QCString trDirRelation(const char *name)
+    virtual QCString trDirRelation(const QCString &name)
     {
       return QCString(name)+" Կապ";
     }
@@ -1740,7 +1742,7 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
      *  table is shown. The heading for the first column mentions the
      *  source file that has a relation to another file.
      */
-    virtual QCString trFileIn(const char *name)
+    virtual QCString trFileIn(const QCString &name)
     {
       return (QCString)"Ֆայլը " + name + " ում";
     }
@@ -1749,7 +1751,7 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
      *  table is shown. The heading for the second column mentions the
      *  destination file that is included.
      */
-    virtual QCString trIncludesFileIn(const char *name)
+    virtual QCString trIncludesFileIn(const QCString &name)
     {
       return (QCString)"Ներառում է ֆայլը " + name + " ում";
     }
@@ -1796,9 +1798,8 @@ class TranslatorArmenian : public TranslatorAdapter_1_8_0
     { return "Հեղինակային իրավունք"; }
 
     /*! Header for the graph showing the directory dependencies */
-    virtual QCString trDirDepGraph(const char *name)
+    virtual QCString trDirDepGraph(const QCString &name)
     { return name + QCString("-ի ֆայլադարանների կախվածությունների գծագիր:"); }
 
 };
-
 #endif
