@@ -133,7 +133,7 @@
 //   2017/10/12 Arnd Weber (arnd dot weber at bafg dot de)
 //              Beatrix Konz
 //    - Updated for 1.8.13
-//    - Resynced trMemberDataDocumentation() and trFileMembers() to include the 
+//    - Resynced trMemberDataDocumentation() and trFileMembers() to include the
 //      boolean switch OPTIMIZE_OUTPUT_FOR_C
 //    - Replaced "\t" by "    "
 //
@@ -143,7 +143,7 @@
 #ifndef TRANSLATOR_DE_H
 #define TRANSLATOR_DE_H
 
-class TranslatorGerman : public Translator
+class TranslatorGerman : public TranslatorAdapter_1_8_15
 {
   public:
 
@@ -166,6 +166,11 @@ class TranslatorGerman : public Translator
       return "\\usepackage[ngerman]{babel}\n";
     }
 
+    virtual QCString trISOLang()
+    {
+      return "de";
+    }
+
     // --- Language translation methods -------------------
 
     /*! used in the compound documentation before a list of related functions. */
@@ -228,9 +233,9 @@ class TranslatorGerman : public Translator
     /*! this is put at the author sections at the bottom of man pages.
      *  parameter s is name of the project name.
      */
-    virtual QCString trGeneratedAutomatically(const char *s)
+    virtual QCString trGeneratedAutomatically(const QCString &s)
     { QCString result="Automatisch erzeugt von Doxygen";
-      if (s) result+=(QCString)" für "+s;
+      if (!s.isEmpty()) result+=(QCString)" für "+s;
       result+=" aus dem Quellcode.";
       return result;
     }
@@ -316,8 +321,17 @@ class TranslatorGerman : public Translator
 
     /*! This is an introduction to the class hierarchy. */
     virtual QCString trClassHierarchyDescription()
-    { return "Die Liste der Ableitungen ist -mit Einschränkungen- "
-             "alphabetisch sortiert:";
+    {
+      if (Config_getBool(OPTIMIZE_OUTPUT_VHDL))
+      {
+        return "Hier folgt eine hierarchische Auflistung der "
+               "Entwurfseinheiten:";
+      }
+      else
+      {
+        return "Die Liste der Ableitungen ist -mit Einschränkungen- "
+               "alphabetisch sortiert:";
+      }
     }
 
     /*! This is an introduction to the list with all files. */
@@ -334,13 +348,17 @@ class TranslatorGerman : public Translator
     {
       if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
-    return "Hier folgt die Aufzählung aller Datenstrukturen "
-           "mit einer Kurzbeschreibung:";
+        return "Hier folgt die Aufzählung aller Datenstrukturen "
+               "mit einer Kurzbeschreibung:";
+      }
+      else if (Config_getBool(OPTIMIZE_OUTPUT_SLICE))
+      {
+        return "Hier folgt die Aufzählung aller Klassen mit einer Kurzbeschreibung:";
       }
       else
       {
-    return "Hier folgt die Aufzählung aller Klassen, Strukturen, "
-           "Varianten und Schnittstellen mit einer Kurzbeschreibung:";
+        return "Hier folgt die Aufzählung aller Klassen, Strukturen, "
+               "Varianten und Schnittstellen mit einer Kurzbeschreibung:";
       }
     }
 
@@ -469,6 +487,10 @@ class TranslatorGerman : public Translator
       {
         return "Datenstruktur-Dokumentation";
       }
+      else if (Config_getBool(OPTIMIZE_OUTPUT_VHDL))
+      {
+          return trDesignUnitDocumentation();
+      }
       else
       {
         return "Klassen-Dokumentation";
@@ -581,16 +603,16 @@ class TranslatorGerman : public Translator
     /*! This is used in the standard footer of each page and indicates when
      *  the page was generated
      */
-    virtual QCString trGeneratedAt(const char *date,const char *projName)
+    virtual QCString trGeneratedAt(const QCString &date,const QCString &projName)
     {
       QCString result=(QCString)"Erzeugt am "+date;
-      if (projName) result+=(QCString)" für "+projName;
+      if (!projName.isEmpty()) result+=(QCString)" für "+projName;
       result+=(QCString)" von";
       return result;
     }
 
     /*! this text is put before a class diagram */
-    virtual QCString trClassDiagram(const char *clName)
+    virtual QCString trClassDiagram(const QCString &clName)
     {
       return (QCString)"Klassendiagramm für "+clName+":";
     }
@@ -669,7 +691,7 @@ class TranslatorGerman : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! used as the title of the HTML page of a class/struct/union */
-    virtual QCString trCompoundReference(const char *clName,
+    virtual QCString trCompoundReference(const QCString &clName,
                                  ClassDef::CompoundType compType,
                                  bool isTemplate)
     {
@@ -691,7 +713,7 @@ class TranslatorGerman : public Translator
     }
 
     /*! used as the title of the HTML page of a file */
-    virtual QCString trFileReference(const char *fileName)
+    virtual QCString trFileReference(const QCString &fileName)
     {
       QCString result=fileName;
       result+="-Dateireferenz";
@@ -699,7 +721,7 @@ class TranslatorGerman : public Translator
     }
 
     /*! used as the title of the HTML page of a namespace */
-    virtual QCString trNamespaceReference(const char *namespaceName)
+    virtual QCString trNamespaceReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+="-Namensbereichsreferenz";
@@ -842,10 +864,11 @@ class TranslatorGerman : public Translator
     virtual QCString trGeneratedFromFiles(ClassDef::CompoundType compType,
         bool single)
     { // single is true implies a single file
+      static bool vhdlOpt = Config_getBool(OPTIMIZE_OUTPUT_VHDL);
       QCString result=(QCString)"Die Dokumentation für diese";
       switch(compType)
       {
-        case ClassDef::Class:      result+=" Klasse"; break;
+        case ClassDef::Class:      result+=vhdlOpt?"Entwurfseinheiten":"Klasse"; break;
         case ClassDef::Struct:     result+=" Struktur"; break;
         case ClassDef::Union:      result+=" Variante"; break;
         case ClassDef::Interface:  result+=" Schnittstelle"; break;
@@ -906,13 +929,13 @@ class TranslatorGerman : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! this text is put before a collaboration diagram */
-    virtual QCString trCollaborationDiagram(const char *clName)
+    virtual QCString trCollaborationDiagram(const QCString &clName)
     {
       return (QCString)"Zusammengehörigkeiten von "+clName+":";
     }
 
     /*! this text is put before an include dependency graph */
-    virtual QCString trInclDepGraph(const char *fName)
+    virtual QCString trInclDepGraph(const QCString &fName)
     {
       return (QCString)"Include-Abhängigkeitsdiagramm für "+fName+":";
     }
@@ -1231,7 +1254,7 @@ class TranslatorGerman : public Translator
     }
 
     /*! Used as the title of a Java package */
-    virtual QCString trPackage(const char *name)
+    virtual QCString trPackage(const QCString &name)
     {
       return (QCString)"Paket "+name;
     }
@@ -1615,7 +1638,7 @@ class TranslatorGerman : public Translator
     /*! This returns the title of a directory page. The name of the
      *  directory is passed via \a dirName.
      */
-    virtual QCString trDirReference(const char *dirName)
+    virtual QCString trDirReference(const QCString &dirName)
     { QCString result=dirName; result+="-Verzeichnisreferenz"; return result; }
 
     /*! This returns the word directory with or without starting capital
@@ -1743,7 +1766,7 @@ class TranslatorGerman : public Translator
     }
 
     /*! used as the title of the HTML page of a module/type (Fortran) */
-    virtual QCString trCompoundReferenceFortran(const char *clName,
+    virtual QCString trCompoundReferenceFortran(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
@@ -1765,7 +1788,7 @@ class TranslatorGerman : public Translator
       return result;
     }
     /*! used as the title of the HTML page of a module (Fortran) */
-    virtual QCString trModuleReference(const char *namespaceName)
+    virtual QCString trModuleReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+="-Modul-Referenz";
@@ -1869,7 +1892,7 @@ class TranslatorGerman : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! directory relation for \a name */
-    virtual QCString trDirRelation(const char *name)
+    virtual QCString trDirRelation(const QCString &name)
     {
       return "Bezug " + QCString(name);
     }
@@ -1906,7 +1929,7 @@ class TranslatorGerman : public Translator
      *  table is shown. The heading for the first column mentions the
      *  source file that has a relation to another file.
      */
-    virtual QCString trFileIn(const char *name)
+    virtual QCString trFileIn(const QCString &name)
     {
       return (QCString)"Datei in "+name;
     }
@@ -1915,7 +1938,7 @@ class TranslatorGerman : public Translator
      *  table is shown. The heading for the second column mentions the
      *  destination file that is included.
      */
-    virtual QCString trIncludesFileIn(const char *name)
+    virtual QCString trIncludesFileIn(const QCString &name)
     {
       return (QCString)"Include-Dateien in "+name;
     }
@@ -1960,7 +1983,7 @@ class TranslatorGerman : public Translator
     { return "Copyright"; }
 
     /*! Header for the graph showing the directory dependencies */
-    virtual QCString trDirDepGraph(const char *name)
+    virtual QCString trDirDepGraph(const QCString &name)
     { return QCString("Diagramm der Verzeichnisabhängigkeiten für ")+name+":"; }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1982,7 +2005,7 @@ class TranslatorGerman : public Translator
     /*! Used file list for a Java enum */
     virtual QCString trEnumGeneratedFromFiles(bool single)
     { QCString result = "Die Dokumentation für diesen enum wurde aus ";
-      if (single) 
+      if (single)
         result += "der folgenden Datei";
       else
         result += "den folgenden Dateien";
@@ -1991,11 +2014,11 @@ class TranslatorGerman : public Translator
     }
 
     /*! Header of a Java enum page (Java enums are represented as classes). */
-    virtual QCString trEnumReference(const char *name)
+    virtual QCString trEnumReference(const QCString &name)
     { return QCString(name)+" Enum-Referenz"; }
 
     /*! Used for a section containing inherited members */
-    virtual QCString trInheritedFrom(const char *members,const char *what)
+    virtual QCString trInheritedFrom(const QCString &members,const QCString &what)
     { return QCString(members)+" geerbt von "+what; }
 
     /*! Header of the sections with inherited members specific for the
@@ -2059,14 +2082,6 @@ class TranslatorGerman : public Translator
       return "Methodendokumentation";
     }
 
-    /*! Used as the title of the design overview picture created for the
-     *  VHDL output.
-     */
-    virtual QCString trDesignOverview()
-    {
-      return "Übersicht";
-    }
-
 //////////////////////////////////////////////////////////////////////////
 // new since 1.8.4
 //////////////////////////////////////////////////////////////////////////
@@ -2084,7 +2099,7 @@ class TranslatorGerman : public Translator
     { return "Konstante Gruppen"; }
 
     /** UNO IDL constant groups */
-    virtual QCString trConstantGroupReference(const char *namespaceName)
+    virtual QCString trConstantGroupReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+=" Konstantengruppen-Referenz";
@@ -2092,7 +2107,7 @@ class TranslatorGerman : public Translator
     }
 
     /** UNO IDL service page title */
-    virtual QCString trServiceReference(const char *sName)
+    virtual QCString trServiceReference(const QCString &sName)
     {
       QCString result=(QCString)sName;
       result+=" Dienstreferenz";
@@ -2100,7 +2115,7 @@ class TranslatorGerman : public Translator
     }
 
     /** UNO IDL singleton page title */
-    virtual QCString trSingletonReference(const char *sName)
+    virtual QCString trSingletonReference(const QCString &sName)
     {
       QCString result=(QCString)sName;
       result+=" Singleton-Referenz";
@@ -2125,6 +2140,129 @@ class TranslatorGerman : public Translator
     }
 
 //////////////////////////////////////////////////////////////////////////
+// new since 1.8.15
+//////////////////////////////////////////////////////////////////////////
+
+    /** VHDL design unit hierarchy */
+    virtual QCString trDesignUnitHierarchy()
+    { return "Entwurfseinheiten-Hierarchie"; }
+    /** VHDL design unit list */
+    virtual QCString trDesignUnitList()
+    { return "Auflistung der Entwurfseinheiten"; }
+    /** VHDL design unit members */
+    virtual QCString trDesignUnitMembers()
+    { return "Entwurfseinheiten-Elemente"; }
+    /** VHDL design unit list description */
+    virtual QCString trDesignUnitListDescription()
+    {
+        return "Hier folgt die Aufzählung aller Entwurfseinheiten und deren "
+               "Elemente mit einer Kurzbeschreibung:";
+    }
+    /** VHDL design unit index */
+    virtual QCString trDesignUnitIndex()
+    { return "Entwurfseinheiten-Verzeichnis"; }
+    /** VHDL design units */
+    virtual QCString trDesignUnits()
+    { return "Entwurfseinheiten"; }
+    /** VHDL functions/procedures/processes */
+    virtual QCString trFunctionAndProc()
+    { return "Funktionen/Prozeduren/Prozesse"; }
+    /** VHDL type */
+    virtual QCString trVhdlType(uint64 type,bool single)
+    {
+      switch(type)
+      {
+        case VhdlDocGen::LIBRARY:
+          if (single) return "Bibliothek";
+          else        return "Bibliotheken";
+        case VhdlDocGen::PACKAGE:
+          if (single) return "Paket";
+          else        return "Pakete";
+        case VhdlDocGen::SIGNAL:
+          if (single) return "Signal";
+          else        return "Signale";
+        case VhdlDocGen::COMPONENT:
+          if (single) return "Komponente";
+          else        return "Komponenten";
+        case VhdlDocGen::CONSTANT:
+          if (single) return "Konstante";
+          else        return "Konstanten";
+        case VhdlDocGen::ENTITY:
+          if (single) return "Entwurfseinheit";
+          else        return "Entwurfseinheiten";
+        case VhdlDocGen::TYPE:
+          if (single) return "Typ";
+          else        return "Typen";
+        case VhdlDocGen::SUBTYPE:
+          if (single) return "Subtyp";
+          else        return "Subtypen";
+        case VhdlDocGen::FUNCTION:
+          if (single) return "Funktion";
+          else        return "Funktionen";
+        case VhdlDocGen::RECORD:
+          if (single) return "Datenstruktur";
+          else        return "Datenstrukturen";
+        case VhdlDocGen::PROCEDURE:
+          if (single) return "Prozedur";
+          else        return "Prozeduren";
+        case VhdlDocGen::ARCHITECTURE:
+          if (single) return "Architektur";
+          else        return "Architekturen";
+        case VhdlDocGen::ATTRIBUTE:
+          if (single) return "Attribut";
+          else        return "Attribute";
+        case VhdlDocGen::PROCESS:
+          if (single) return "Prozess";
+          else        return "Prozesse";
+        case VhdlDocGen::PORT:
+          if (single) return "Schnittstelle";
+          else        return "Schnittstellen";
+        case VhdlDocGen::USE:
+          if (single) return "Use Klausel";
+          else        return "Use Klauseln";
+        case VhdlDocGen::GENERIC:
+          if (single) return "Parameter";
+          else        return "Parameter";
+        case VhdlDocGen::PACKAGE_BODY:
+          return "Paketkörper";
+        case VhdlDocGen::UNITS:
+          return "Einheiten";
+        case VhdlDocGen::SHAREDVARIABLE:
+          if (single) return "Geteilte Variable";
+          else        return "Geteilte Variablen";
+        case VhdlDocGen::VFILE:
+          if (single) return "Datei";
+          else        return "Dateien";
+        case VhdlDocGen::GROUP:
+          if (single) return "Gruppe";
+          else        return "Gruppen";
+        case VhdlDocGen::INSTANTIATION:
+          if (single) return "Instanziierung";
+          else        return "Instanziierungen";
+        case VhdlDocGen::ALIAS:
+          if (single) return "Alias";
+          else        return "Aliase";
+        case VhdlDocGen::CONFIG:
+          if (single) return "Konfiguration";
+          else        return "Konfigurationen";
+        case VhdlDocGen::MISCELLANEOUS:
+          return "Verschiedenes";
+        case VhdlDocGen::UCF_CONST:
+          return "Constraints";
+        default:
+          return "Klasse";
+      }
+    }
+    virtual QCString trCustomReference(const QCString &name)
+    { return QCString(name)+"-Referenz"; }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.19
+//////////////////////////////////////////////////////////////////////////
+
+    /** VHDL design unit documentation */
+    virtual QCString trDesignUnitDocumentation()
+    { return "Entwurfseinheiten-Dokumentation"; }
 
 };
 
