@@ -2,7 +2,7 @@
  *
  *
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -35,20 +35,20 @@ class TranslatorJapaneseEn : public TranslatorEnglish
     virtual QCString idLanguage()
     { return "japanese-en"; }
     virtual QCString latexLanguageSupportCommand()
-      {
-	return "platex";
-      }
-    /*! returns the name of the package that is included by LaTeX */
-    virtual QCString idLanguageCharset()
     {
-      if (Config_getBool("USE_WINDOWS_ENCODING"))
-      {
-        return "Shift_JIS";
-      }
-      else
-      {
-        return "euc-jp";
-      }
+      return "\\usepackage{CJKutf8}\n";
+    }
+    virtual QCString latexFontenc()
+    {
+      return "";
+    }
+    virtual QCString latexDocumentPre()
+    {
+      return "\\begin{CJK}{UTF8}{min}\n";
+    }
+    virtual QCString latexDocumentPost()
+    {
+      return "\\end{CJK}\n";
     }
     virtual QCString trRTFansicp()
     {
@@ -62,6 +62,11 @@ class TranslatorJapaneseEn : public TranslatorEnglish
     {
       return "128";
     }
+
+    virtual QCString trISOLang()
+    {
+      return "ja";
+    }
 };
 
 #endif
