@@ -42,7 +42,7 @@
 #define TRANSLATOR_RO_H
 
 
-class TranslatorRomanian : public Translator
+class TranslatorRomanian : public TranslatorAdapter_1_8_15
 {
   public:
 
@@ -74,6 +74,10 @@ class TranslatorRomanian : public Translator
     {
       return "\\usepackage[romanian]{babel}\n";
     }
+    virtual QCString trISOLang()
+    {
+      return "ro";
+    }
 
     // --- Language translation methods -------------------
 
@@ -104,7 +108,7 @@ class TranslatorRomanian : public Translator
     /*! header that is put before the list of member attributes. */
     virtual QCString trMemberDataDocumentation()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Documentaţia Câmpurilor";
       }
@@ -138,9 +142,9 @@ class TranslatorRomanian : public Translator
     /*! this is put at the author sections at the bottom of man pages.
      *  parameter s is name of the project name.
      */
-    virtual QCString trGeneratedAutomatically(const char *s)
+    virtual QCString trGeneratedAutomatically(const QCString &s)
     { QCString result="Generat automat de Doxygen";
-      if (s) result+=(QCString)" pentru "+s;
+      if (!s.isEmpty()) result+=(QCString)" pentru "+s;
       result+=" din codul sursă.";
       return result;
     }
@@ -172,7 +176,7 @@ class TranslatorRomanian : public Translator
     /*! This is put above each page as a link to the list of annotated classes */
     virtual QCString trCompoundList()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Structuri de Date";
       }
@@ -190,7 +194,7 @@ class TranslatorRomanian : public Translator
     /*! This is put above each page as a link to all members of compounds. */
     virtual QCString trCompoundMembers()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Câmpurile de Date";
       }
@@ -204,7 +208,7 @@ class TranslatorRomanian : public Translator
     /*! This is put above each page as a link to all members of files. */
     virtual QCString trFileMembers()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Globale";
       }
@@ -246,10 +250,14 @@ class TranslatorRomanian : public Translator
     /*! This is an introduction to the annotated compound list. */
     virtual QCString trCompoundListDescription()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Lista structurilor de date, cu scurte descrieri:";
       }
+      else if (Config_getBool(OPTIMIZE_OUTPUT_SLICE))
+      {
+        return "Lista claselor, cu scurte descrieri:";
+      }
       else
       {
         return "Lista claselor, structurilor, uniunilor şi interfeţelor"
@@ -263,7 +271,7 @@ class TranslatorRomanian : public Translator
     {
       QCString result="Lista tuturor ";
 
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result+="câmpurilor ";
         if (!extractAll) result+=" documentate ";
@@ -278,7 +286,7 @@ class TranslatorRomanian : public Translator
       result+=", cu legături către ";
       if (!extractAll)
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result+="documentaţia structurii/uniunii pentru fiecare câmp în parte:";
         }
@@ -289,7 +297,7 @@ class TranslatorRomanian : public Translator
       }
       else
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result+="structurile/uniunile de care aparţin:";
         }
@@ -306,7 +314,7 @@ class TranslatorRomanian : public Translator
     virtual QCString trFileMembersDescription(bool extractAll)
     {
       QCString result="Lista tuturor ";
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result+="funcţiilor, variabilelor, definiţiilor, enumerărilor şi definiţiilor de tip";
 	  if (!extractAll) result+=" documentate";
@@ -362,7 +370,7 @@ class TranslatorRomanian : public Translator
      */
     virtual QCString trCompoundIndex()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Indexul Structurilor de Date";
       }
@@ -390,7 +398,7 @@ class TranslatorRomanian : public Translator
      */
     virtual QCString trClassDocumentation()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Documentaţia Structurilor de Date";
       }
@@ -494,7 +502,7 @@ class TranslatorRomanian : public Translator
      */
     virtual QCString trCompounds()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Structuri de Date";
       }
@@ -508,16 +516,16 @@ class TranslatorRomanian : public Translator
     /*! This is used in the standard footer of each page and indicates when
      *  the page was generated
      */
-    virtual QCString trGeneratedAt(const char *date,const char *projName)
+    virtual QCString trGeneratedAt(const QCString &date,const QCString &projName)
     {
       QCString result=(QCString)"Generat "+date;
-      if (projName) result+=(QCString)" pentru "+projName;
+      if (!projName.isEmpty()) result+=(QCString)" pentru "+projName;
       result+=(QCString)" de către";
       return result;
     }
 
     /*! this text is put before a class diagram */
-    virtual QCString trClassDiagram(const char *clName)
+    virtual QCString trClassDiagram(const QCString &clName)
     {
       return (QCString)"Diagrama de relaţii pentru "+clName;
     }
@@ -595,7 +603,7 @@ class TranslatorRomanian : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! used as the title of the HTML page of a class/struct/union */
-    virtual QCString trCompoundReference(const char *clName,
+    virtual QCString trCompoundReference(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
@@ -618,7 +626,7 @@ class TranslatorRomanian : public Translator
     }
 
     /*! used as the title of the HTML page of a file */
-    virtual QCString trFileReference(const char *fileName)
+    virtual QCString trFileReference(const QCString &fileName)
     {
       QCString result="Referinţă la fişierul ";
       result+=fileName;
@@ -626,7 +634,7 @@ class TranslatorRomanian : public Translator
     }
 
     /*! used as the title of the HTML page of a namespace */
-    virtual QCString trNamespaceReference(const char *namespaceName)
+    virtual QCString trNamespaceReference(const QCString &namespaceName)
     {
       QCString result="Referinţă la Namespace-ul ";
       result+=namespaceName;
@@ -825,12 +833,12 @@ class TranslatorRomanian : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! this text is put before a collaboration diagram */
-    virtual QCString trCollaborationDiagram(const char *clName)
+    virtual QCString trCollaborationDiagram(const QCString &clName)
     {
       return (QCString)"Diagrama de relaţii pentru "+clName+":";
     }
     /*! this text is put before an include dependency graph */
-    virtual QCString trInclDepGraph(const char *fName)
+    virtual QCString trInclDepGraph(const QCString &fName)
     {
       return (QCString)"Graful dependenţelor prin incluziune pentru "+fName+":";
     }
@@ -905,7 +913,7 @@ class TranslatorRomanian : public Translator
     }
     virtual QCString trPublicAttribs()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Câmpuri de Date";
       }
@@ -1097,7 +1105,7 @@ class TranslatorRomanian : public Translator
     /*! Used for Java classes in the summary section of Java packages */
     virtual QCString trClasses()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Structuri de Date";
       }
@@ -1107,7 +1115,7 @@ class TranslatorRomanian : public Translator
       }
     }
     /*! Used as the title of a Java package */
-    virtual QCString trPackage(const char *name)
+    virtual QCString trPackage(const QCString &name)
     {
       return (QCString)"Pachet "+name;
     }
@@ -1501,7 +1509,7 @@ class TranslatorRomanian : public Translator
      /*! This returns the title of a directory page. The name of the
       *  directory is passed via \a dirName.
       */
-     virtual QCString trDirReference(const char *dirName)
+     virtual QCString trDirReference(const QCString &dirName)
      {
        QCString  result="Director-referinţă "; result+=dirName;
        return result;
@@ -1631,7 +1639,7 @@ class TranslatorRomanian : public Translator
     }
 
     /*! used as the title of the HTML page of a module/type (Fortran) */
-    virtual QCString trCompoundReferenceFortran(const char *clName,
+    virtual QCString trCompoundReferenceFortran(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
@@ -1652,7 +1660,7 @@ class TranslatorRomanian : public Translator
       return result;
     }
     /*! used as the title of the HTML page of a module (Fortran) */
-    virtual QCString trModuleReference(const char *namespaceName)
+    virtual QCString trModuleReference(const QCString &namespaceName)
     {
       QCString result="Referinţă la Modulul ";
 	  result += namespaceName;
@@ -1754,7 +1762,7 @@ class TranslatorRomanian : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! directory relation for \a name */
-    virtual QCString trDirRelation(const char *name)
+    virtual QCString trDirRelation(const QCString &name)
     {
       return QCString(name)+" Relație";
     }
@@ -1791,7 +1799,7 @@ class TranslatorRomanian : public Translator
      *  table is shown. The heading for the first column mentions the
      *  source file that has a relation to another file.
      */
-    virtual QCString trFileIn(const char *name)
+    virtual QCString trFileIn(const QCString &name)
     {
       return (QCString)"Fișierul din "+name;
     }
@@ -1800,7 +1808,7 @@ class TranslatorRomanian : public Translator
      *  table is shown. The heading for the second column mentions the
      *  destination file that is included.
      */
-    virtual QCString trIncludesFileIn(const char *name)
+    virtual QCString trIncludesFileIn(const QCString &name)
     {
       return (QCString)"Include fișierul din "+name;
     }
@@ -1845,7 +1853,7 @@ class TranslatorRomanian : public Translator
     { return "Copyright"; }
 
     /*! Header for the graph showing the directory dependencies */
-    virtual QCString trDirDepGraph(const char *name)
+    virtual QCString trDirDepGraph(const QCString &name)
     { return QCString("Grafic de dependență a directoarelor pentru ")+name+":"; }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1875,11 +1883,11 @@ class TranslatorRomanian : public Translator
     }
 
     /*! Header of a Java enum page (Java enums are represented as classes). */
-    virtual QCString trEnumReference(const char *name)
+    virtual QCString trEnumReference(const QCString &name)
     { return QCString(name)+" Referință Enum"; }
 
     /*! Used for a section containing inherited members */
-    virtual QCString trInheritedFrom(const char *members,const char *what)
+    virtual QCString trInheritedFrom(const QCString &members,const QCString &what)
     { return QCString(members)+" moștenit(e) din "+what; }
 
     /*! Header of the sections with inherited members specific for the
@@ -1908,7 +1916,7 @@ class TranslatorRomanian : public Translator
      */
     virtual QCString trProvidedByCategory()
     {
-      return "Furnizat de categoria @1.";
+      return "Furnizat de categoria @0.";
     }
 
     /*! Used in a method of an Objective-C category that extends a class.
@@ -1917,7 +1925,7 @@ class TranslatorRomanian : public Translator
      */
     virtual QCString trExtendsClass()
     {
-      return "Extinde clasa @1.";
+      return "Extinde clasa @0.";
     }
 
     /*! Used as the header of a list of class methods in Objective-C.
@@ -1943,14 +1951,6 @@ class TranslatorRomanian : public Translator
       return "Documentația Metodelor";
     }
 
-    /*! Used as the title of the design overview picture created for the
-     *  VHDL output.
-     */
-    virtual QCString trDesignOverview()
-    {
-      return "Vedere de Ansamblu a Designului";
-    }
-
 //////////////////////////////////////////////////////////////////////////
 // new since 1.8.4
 //////////////////////////////////////////////////////////////////////////
@@ -1968,21 +1968,21 @@ class TranslatorRomanian : public Translator
     { return "Grupuri Constante"; }
 
     /** UNO IDL constant groups */
-    virtual QCString trConstantGroupReference(const char *namespaceName)
+    virtual QCString trConstantGroupReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+=" Referință Grup Constant";
       return result;
     }
     /** UNO IDL service page title */
-    virtual QCString trServiceReference(const char *sName)
+    virtual QCString trServiceReference(const QCString &sName)
     {
       QCString result=(QCString)sName;
       result+=" Referință Serviciu";
       return result;
     }
     /** UNO IDL singleton page title */
-    virtual QCString trSingletonReference(const char *sName)
+    virtual QCString trSingletonReference(const QCString &sName)
     {
       QCString result=(QCString)sName;
       result+=" Referință Singleton";
@@ -2013,8 +2013,6 @@ class TranslatorRomanian : public Translator
       return result;
     }
 
-//////////////////////////////////////////////////////////////////////////
-
 };
 
 #endif
