@@ -32,7 +32,7 @@
 #ifndef TRANSLATOR_SK_H
 #define TRANSLATOR_SK_H
 
-class TranslatorSlovak : public Translator
+class TranslatorSlovak : public TranslatorAdapter_1_8_15
 {
   public:
     // --- Language control methods -------------------
@@ -41,8 +41,18 @@ class TranslatorSlovak : public Translator
     { return "slovak"; }
 
     virtual QCString latexLanguageSupportCommand()
-    { return "\\usepackage[slovak]{babel}\n"; }
-
+    { return "\\usepackage[slovak]{babel}\n"
+             "\\usepackage{regexpatch}\n"
+             "\\makeatletter\n"
+             "% Change the `-` delimiter to an active character\n"
+             "\\xpatchparametertext\\@@@cmidrule{-}{\\cA-}{}{}\n"
+             "\\xpatchparametertext\\@cline{-}{\\cA-}{}{}\n"
+             "\\makeatother\n";
+    }
+    virtual QCString trISOLang()
+    {
+      return "sk";
+    }
     // --- Language translation methods -------------------
 
     /*! used in the compound documentation before a list of related functions. */
@@ -72,7 +82,7 @@ class TranslatorSlovak : public Translator
     /*! header that is put before the list of member attributes. */
     virtual QCString trMemberDataDocumentation()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Dokumentácia k položkám";
       }
@@ -105,10 +115,10 @@ class TranslatorSlovak : public Translator
     /*! this is put at the author sections at the bottom of man pages.
      *	parameter s is name of the project name.
      */
-    virtual QCString trGeneratedAutomatically(const char *s)
+    virtual QCString trGeneratedAutomatically(const QCString &s)
     { QCString result("Generované automaticky programom Doxygen "
                       "zo zdrojových textov");
-      if (s)
+      if (!s.isEmpty())
           result+=(QCString)" projektu "+s;
       result+=".";
       return result;
@@ -141,7 +151,7 @@ class TranslatorSlovak : public Translator
     /*! This is put above each page as a link to the list of annotated classes */
     virtual QCString trCompoundList()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Dátové štruktúry";
       }
@@ -158,7 +168,7 @@ class TranslatorSlovak : public Translator
     /*! This is put above each page as a link to all members of compounds. */
     virtual QCString trCompoundMembers()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Dátové položky";
       }
@@ -171,7 +181,7 @@ class TranslatorSlovak : public Translator
     /*! This is put above each page as a link to all members of files. */
     virtual QCString trFileMembers()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Globálne symboly";
       }
@@ -211,7 +221,7 @@ class TranslatorSlovak : public Translator
     /*! This is an introduction to the annotated compound list. */
     virtual QCString trCompoundListDescription()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Nasledujúci zoznam obsahuje identifikáciu dátových "
                "štruktúr a ich stručné popisy:";
@@ -235,7 +245,7 @@ class TranslatorSlovak : public Translator
         result += "dokumentovaných ";
       }
 
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result += "položiek štruktúr (struct) a unionov (union) ";
       }
@@ -248,7 +258,7 @@ class TranslatorSlovak : public Translator
 
       if (!extractAll)
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result += "dokumentáciu štruktúr/unionov, ku ktorým prislúchajú:";
         }
@@ -259,7 +269,7 @@ class TranslatorSlovak : public Translator
       }
       else
       {
-        if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+        if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
         {
           result+="štruktúry/uniony, ku ktorým prislúchajú:";
         }
@@ -278,7 +288,7 @@ class TranslatorSlovak : public Translator
       QCString result="Tu nájdete zoznam všetkých ";
       if (!extractAll) result+="dokumentovaných ";
 
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         result+="funkcií, premenných, makier, enumerácií a definícií typov (typedef) "
                 "s odkazmi na ";
@@ -334,7 +344,7 @@ class TranslatorSlovak : public Translator
      */
     virtual QCString trCompoundIndex()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Register dátových štruktúr";
       }
@@ -455,7 +465,7 @@ class TranslatorSlovak : public Translator
      */
     virtual QCString trCompounds()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Dátové štruktúry";
       }
@@ -468,16 +478,16 @@ class TranslatorSlovak : public Translator
     /*! This is used in the standard footer of each page and indicates when
      *	the page was generated
      */
-    virtual QCString trGeneratedAt(const char *date,const char *projName)
+    virtual QCString trGeneratedAt(const QCString &date,const QCString &projName)
     {
       QCString result=(QCString)"Generované "+date;
-      if (projName) result+=(QCString)" pre projekt "+projName;
+      if (!projName.isEmpty()) result+=(QCString)" pre projekt "+projName;
       result+=(QCString)" programom";
       return result;
     }
 
     /*! this text is put before a class diagram */
-    virtual QCString trClassDiagram(const char *clName)
+    virtual QCString trClassDiagram(const QCString &clName)
     {
       return (QCString)"Diagram dedičnosti pre triedu "+clName;
     }
@@ -554,7 +564,7 @@ class TranslatorSlovak : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! used as the title of the HTML page of a class/struct/union */
-    virtual QCString trCompoundReference(const char *clName,
+    virtual QCString trCompoundReference(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
@@ -576,7 +586,7 @@ class TranslatorSlovak : public Translator
     }
 
     /*! used as the title of the HTML page of a file */
-    virtual QCString trFileReference(const char *fileName)
+    virtual QCString trFileReference(const QCString &fileName)
     {
       QCString result("Dokumentácia súboru ");
       result+=fileName;
@@ -584,7 +594,7 @@ class TranslatorSlovak : public Translator
     }
 
     /*! used as the title of the HTML page of a namespace */
-    virtual QCString trNamespaceReference(const char *namespaceName)
+    virtual QCString trNamespaceReference(const QCString &namespaceName)
     {
       QCString result("Dokumentácia priestoru mien ");
       result+=namespaceName;
@@ -796,12 +806,12 @@ class TranslatorSlovak : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! this text is put before a collaboration diagram */
-    virtual QCString trCollaborationDiagram(const char *clName)
+    virtual QCString trCollaborationDiagram(const QCString &clName)
     {
       return (QCString)"Diagram tried pre "+clName+":";
     }
     /*! this text is put before an include dependency graph */
-    virtual QCString trInclDepGraph(const char *fName)
+    virtual QCString trInclDepGraph(const QCString &fName)
     {
       return (QCString)"Graf závislostí na vkladaných súboroch "
                     "pre "+fName+":";
@@ -877,7 +887,7 @@ class TranslatorSlovak : public Translator
     }
     virtual QCString trPublicAttribs()
     {
-      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
+      if (Config_getBool(OPTIMIZE_OUTPUT_FOR_C))
       {
         return "Dátové položky";
       }
@@ -1080,7 +1090,7 @@ class TranslatorSlovak : public Translator
       return "Triedy";
     }
     /*! Used as the title of a Java package */
-    virtual QCString trPackage(const char *name)
+    virtual QCString trPackage(const QCString &name)
     {
       return (QCString)"Balík "+name;
     }
@@ -1452,7 +1462,7 @@ class TranslatorSlovak : public Translator
     /*! This returns the title of a directory page. The name of the
      *  directory is passed via \a dirName.
      */
-    virtual QCString trDirReference(const char *dirName)
+    virtual QCString trDirReference(const QCString &dirName)
     {
         QCString result = "Referencia k adresáru ";
         result += dirName;
@@ -1585,7 +1595,7 @@ class TranslatorSlovak : public Translator
     }
 
     /*! used as the title of the HTML page of a module/type (Fortran) */
-    virtual QCString trCompoundReferenceFortran(const char *clName,
+    virtual QCString trCompoundReferenceFortran(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
@@ -1607,7 +1617,7 @@ class TranslatorSlovak : public Translator
 
     }
     /*! used as the title of the HTML page of a module (Fortran) */
-    virtual QCString trModuleReference(const char *namespaceName)
+    virtual QCString trModuleReference(const QCString &namespaceName)
     {
       QCString result="Dokumentácia modulu ";
       result += namespaceName;
@@ -1707,7 +1717,7 @@ class TranslatorSlovak : public Translator
 //////////////////////////////////////////////////////////////////////////
 
     /*! directory relation for \a name */
-    virtual QCString trDirRelation(const char *name)
+    virtual QCString trDirRelation(const QCString &name)
     {
       return "Relácia " + QCString(name);
     }
@@ -1744,7 +1754,7 @@ class TranslatorSlovak : public Translator
      *  table is shown. The heading for the first column mentions the
      *  source file that has a relation to another file.
      */
-    virtual QCString trFileIn(const char *name)
+    virtual QCString trFileIn(const QCString &name)
     {
       return (QCString)"Súbor v "+name;
     }
@@ -1753,7 +1763,7 @@ class TranslatorSlovak : public Translator
      *  table is shown. The heading for the second column mentions the
      *  destination file that is included.
      */
-    virtual QCString trIncludesFileIn(const char *name)
+    virtual QCString trIncludesFileIn(const QCString &name)
     {
       return (QCString)"Vkladá (include) súbor z "+name;
     }
@@ -1798,7 +1808,7 @@ class TranslatorSlovak : public Translator
     { return "Copyright"; }
 
     /*! Header for the graph showing the directory dependencies */
-    virtual QCString trDirDepGraph(const char *name)
+    virtual QCString trDirDepGraph(const QCString &name)
     { return QCString("Graf závislosti na priečinkoch pre  ")+name+":"; }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1817,7 +1827,7 @@ class TranslatorSlovak : public Translator
     virtual QCString trAndMore(const QCString &number)
     {
         QCString result("a " + number + " ďaľší");
-        if (atoi(number) >= 5)
+        if (number.toInt() >= 5)
             result += "ch";
         return result + "...";
     }
@@ -1833,11 +1843,11 @@ class TranslatorSlovak : public Translator
     }
 
     /*! Header of a Java enum page (Java enums are represented as classes). */
-    virtual QCString trEnumReference(const char *name)
+    virtual QCString trEnumReference(const QCString &name)
     { return "Referencia k enumerácii "+QCString(name); }
 
     /*! Used for a section containing inherited members */
-    virtual QCString trInheritedFrom(const char *members,const char *what)
+    virtual QCString trInheritedFrom(const QCString &members,const QCString &what)
     { return QCString(members)+" dedí sa z "+what; }
 
     /*! Header of the sections with inherited members specific for the
@@ -1866,7 +1876,7 @@ class TranslatorSlovak : public Translator
      */
     virtual QCString trProvidedByCategory()
     {
-      return "Deklarované v kategórii @1.";
+      return "Deklarované v kategórii @0.";
     }
 
     /*! Used in a method of an Objective-C category that extends a class.
@@ -1875,7 +1885,7 @@ class TranslatorSlovak : public Translator
      */
     virtual QCString trExtendsClass()
     {
-      return "Rozširuje triedu @1.";
+      return "Rozširuje triedu @0.";
     }
 
     /*! Used as the header of a list of class methods in Objective-C.
@@ -1901,14 +1911,6 @@ class TranslatorSlovak : public Translator
       return "Dokumentácia metódy";
     }
 
-    /*! Used as the title of the design overview picture created for the
-     *  VHDL output.
-     */
-    virtual QCString trDesignOverview()
-    {
-      return "Návrhová schéma";
-    }
-
 //////////////////////////////////////////////////////////////////////////
 // new since 1.8.4
 //////////////////////////////////////////////////////////////////////////
@@ -1926,21 +1928,21 @@ class TranslatorSlovak : public Translator
     { return "Konštantné skupiny"; }
 
     /** UNO IDL constant groups */
-    virtual QCString trConstantGroupReference(const char *namespaceName)
+    virtual QCString trConstantGroupReference(const QCString &namespaceName)
     {
       QCString result="Konštantné skupiny z ";
       result += namespaceName;
       return result;
     }
     /** UNO IDL service page title */
-    virtual QCString trServiceReference(const char *sName)
+    virtual QCString trServiceReference(const QCString &sName)
     {
       QCString result="Popis služby ";
       result += sName;
       return result;
     }
     /** UNO IDL singleton page title */
-    virtual QCString trSingletonReference(const char *sName)
+    virtual QCString trSingletonReference(const QCString &sName)
     {
       QCString result="Popis singletonu ";
       result += sName;
@@ -1965,7 +1967,6 @@ class TranslatorSlovak : public Translator
       return result;
     }
 
-//////////////////////////////////////////////////////////////////////////
 };
 
 #endif // TRANSLATOR_SK_H
