@@ -77,10 +77,28 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
      * the empty string is returned in this implementation.
      */
     virtual QCString latexLanguageSupportCommand()
+    {
+      return
+        "\\usepackage[vietnamese]{babel}\n";
+    }
+    virtual QCString latexFontenc()
     {
       return "";
     }
-
+    virtual QCString latexCommandName()
+    {
+      QCString latex_command = Config_getString(LATEX_CMD_NAME);
+      if (latex_command.isEmpty()) latex_command = "latex";
+      if (Config_getBool(USE_PDFLATEX))
+      {
+        if (latex_command == "latex") latex_command = "xelatex";
+      }
+      return latex_command;
+    }
+    virtual QCString trISOLang()
+    {
+      return "vi";
+    }
     // --- Language translation methods -------------------
 
     /*! used in the compound documentation before a list of related functions. */
@@ -143,9 +161,9 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
     /*! this is put at the author sections at the bottom of man pages.
      *  parameter s is name of the project name.
      */
-    virtual QCString trGeneratedAutomatically(const char *s)
+    virtual QCString trGeneratedAutomatically(const QCString &s)
     { QCString result="Được tạo ra bởi Doxygen";
-      if (s) result+=(QCString)" cho "+s;
+      if (!s.isEmpty()) result+=(QCString)" cho "+s;
       result+=" từ mã nguồn.";
       return result;
     }
@@ -252,6 +270,10 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
       {
         return "Đây là cấu trúc cơ sở dữ liệu với mô tả tóm tắt:";
       }
+      else if (Config_getBool(OPTIMIZE_OUTPUT_SLICE))
+      {
+        return "Đây là các classes với các mô tả tóm tắt:";
+      }
       else
       {
         return "Đây là các classes, structs, "
@@ -501,16 +523,16 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
     /*! This is used in the standard footer of each page and indicates when
      *  the page was generated
      */
-    virtual QCString trGeneratedAt(const char *date,const char *projName)
+    virtual QCString trGeneratedAt(const QCString &date,const QCString &projName)
     {
       QCString result=(QCString)"Được biên soạn vào "+date;
-      if (projName) result+=(QCString)" cho mã nguồn dự án "+projName;
+      if (!projName.isEmpty()) result+=(QCString)" cho mã nguồn dự án "+projName;
       result+=(QCString)" bởi";
       return result;
     }
 
     /*! this text is put before a class diagram */
-    virtual QCString trClassDiagram(const char *clName)
+    virtual QCString trClassDiagram(const QCString &clName)
     {
       return (QCString)"Sơ đồ kế thừa cho "+clName+":";
     }
@@ -589,7 +611,7 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
 //////////////////////////////////////////////////////////////////////////
 
     /*! used as the title of the HTML page of a class/struct/union */
-    virtual QCString trCompoundReference(const char *clName,
+    virtual QCString trCompoundReference(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
@@ -611,7 +633,7 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
     }
 
     /*! used as the title of the HTML page of a file */
-    virtual QCString trFileReference(const char *fileName)
+    virtual QCString trFileReference(const QCString &fileName)
     {
       QCString result=fileName;
       result+=" File Tham chiếu";
@@ -619,7 +641,7 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
     }
 
     /*! used as the title of the HTML page of a namespace */
-    virtual QCString trNamespaceReference(const char *namespaceName)
+    virtual QCString trNamespaceReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+=" Namespace Tham chiếu";
@@ -815,12 +837,12 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
 //////////////////////////////////////////////////////////////////////////
 
     /*! this text is put before a collaboration diagram */
-    virtual QCString trCollaborationDiagram(const char *clName)
+    virtual QCString trCollaborationDiagram(const QCString &clName)
     {
       return (QCString)"Sơ đồ liên kết cho "+clName+":";
     }
     /*! this text is put before an include dependency graph */
-    virtual QCString trInclDepGraph(const char *fName)
+    virtual QCString trInclDepGraph(const QCString &fName)
     {
       return (QCString)"Kèm theo graph phụ thuộc cho "+fName+":";
     }
@@ -1100,7 +1122,7 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
       }
     }
     /*! Used as the title of a Java package */
-    virtual QCString trPackage(const char *name)
+    virtual QCString trPackage(const QCString &name)
     {
       return (QCString)"Gói "+name;
     }
@@ -1494,7 +1516,7 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
     /*! This returns the title of a directory page. The name of the
      *  directory is passed via \a dirName.
      */
-    virtual QCString trDirReference(const char *dirName)
+    virtual QCString trDirReference(const QCString &dirName)
     { QCString result=dirName; result+=" Tham chiếu thư mục"; return result; }
 
     /*! This returns the word directory with or without starting capital
@@ -1621,7 +1643,7 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
     }
 
     /*! used as the title of the HTML page of a module/type (Fortran) */
-    virtual QCString trCompoundReferenceFortran(const char *clName,
+    virtual QCString trCompoundReferenceFortran(const QCString &clName,
                                     ClassDef::CompoundType compType,
                                     bool isTemplate)
     {
@@ -1642,7 +1664,7 @@ class TranslatorVietnamese : public TranslatorAdapter_1_6_0
       return result;
     }
     /*! used as the title of the HTML page of a module (Fortran) */
-    virtual QCString trModuleReference(const char *namespaceName)
+    virtual QCString trModuleReference(const QCString &namespaceName)
     {
       QCString result=namespaceName;
       result+=" Tham chiếu Module";
